{***************************************}
{                                       }
{     TVGraphic Library EditDEMO.PAS    }
{        RICHARD P. ANDRESEN            }
{                                       }
{***************************************}

{This demo program illustrates the use of the TVGraphic library.
 It is part of the documentation of TVGraphic.

 It shows the use of the GEditors unit complete with
 Wordwrap plus a Microsft Windows like pull down menu
  - click the window Close Icon or Alt-spacebar.
 (Any type of TVGraphic window can use the pull down menu.)

 Be sure to read about Editor unit memory issues in
 Chapter 15 of the Turbo Vision Programming guide.
 An alternate approach to memory usage is outlined
 in file EDITORS.DOC.
 Parts of TDemoApp.Init specific to using the Editors
 unit are identified with a '' symbol.

 Non-Registered versions of TVGraphic have a demo version
 of the Editors unit - edited memos and files can not be saved.

See TVGDemo1 for a full featured demo program.

Note: You should replace the default DOS critical error handling
routine (which always returns "Abort") for any program you will use
alot or distribute.  See TVGDemo1.

}

program EditDemo;

{$F+,X+,V-}    {+X - use Extended syntax so can call a function as if
                  it were a procedure.}

uses DOS, Memory, MyGraph, GObjects, GDrivers,
     MCursor, GMenus,
     GViews,  GDialogs, GMsgBox, GStdDlg,
     GApp, GWindow, GHistLst, GEditors;

const
  ProgName = 'EditDemo';
  Ver      = '2.00';

  WinNum : integer = 1;
  cmVersion        = 1116;

var
  OldExitProc : Pointer;  { Saves exit procedure address }
  Graphic  : boolean;     { true if screen is in graphic mode }


procedure GExitProc; far;     {must be Far}
  {Exit procedure - restore screen to text mode if program halts}
begin
  ExitProc := OldExitProc; { Restore exit procedure address }
  CloseGraph;              { Shut down the graphics system }
end;


{--------------------------------}
type
  TDemoApp = object(TProgram)
    constructor Init;
    destructor Done; virtual;
    procedure CreateClipBoard;
    function  CreateEdWinMenu : PView;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitMenuBar; virtual;
    procedure InitStatusLine; virtual;
    procedure IntroScreen;
  end;


constructor TDemoApp.Init;
var
  GraphDriver,GraphMode,ErrorCode : integer;
 S: TCommandSet;
begin
  Graphic := false;         {do first}

 MaxHeapSize := 2048;  {MUST set if using Editors unit !!!!}
                        {MUST come before allocating ANY HEAP MEMORY}

  InitMemory;
  InitVideo;
  InitEvents;
  InitSysError;
  InitHistory;

     {register linked in EGA/VGA screen driver}
  RegisterBGIdriver(@EGAVGADriverProc);

  GraphDriver := Detect;                { use autodetection }
     {verify graphics mode}
  DetectGraph(GraphDriver, GraphMode);
  if not ((GraphDriver = VGA) or (GraphDriver = EGA)) then begin
    Done;
    Writeln('Error - system does not support EGA or VGA graphics.');
    Halt(1);
  end;

    {enter graphics mode}
  if GraphDriver = VGA then GraphMode := VGAHi
    else GraphMode := EGAHi;
  InitGraph(GraphDriver,GraphMode,'');

  ErrorCode := GraphResult;
  if ErrorCode <> grOK then begin
    Done;
    Writeln('Graphics Error: ',GraphErrorMsg(ErrorCode));
    Halt(1);
  end
  else begin
          {install exit proc to Close graphics}
    OldExitProc := ExitProc;                { save previous exit proc }
    ExitProc := @GExitProc;                { insert our exit proc in chain }
    Graphic := true;
          {install graphic mode DOS critical error handler}
    (* SysErrorFunc := GSystemError; *)
          {improves look of dark gray and brown on VGA monitors,
           no effect in EGA}
    ImprovePaletteColors;
  end;


  TProgram.Init;   {MCur mouse initialized in TProgram.Init}
      {sets default Viewport to just cover the DeskTop.
       The MainMenu,MessageBar and StatusLine temporarily
       reset Viewport when they draw themselves.}


      {following items may be different for your program}

  MCur.SetSpeed(12,12);  {how fast cursor moves, "normal" is 8,8}
  DoubleDelay := 6;   {time between mouse button presses for double press}
                      {TV uses 8 - very slow}

{Disable menu editing commands}
 S := [cmUndo, cmCut, cmCopy, cmPaste, cmClear, cmSave, cmSaveAs];
 DisableCommands(S);

  IntroScreen;
end;

destructor TDemoApp.Done;
begin
  if Graphic then begin
    TProgram.Done;         {calls MCur.Done}
    CloseGraph;
    Graphic := false;
  end;

  DoneHistory;
  DoneSysError;
  DoneEvents;
  DoneVideo;
  DoneMemory;
end;

procedure TDemoApp.CreateClipBoard;
var
  R : TRect;
  ClipWin : PEditWindow;
begin
  R.A.x := 10 * Charlen;    R.B.x := R.A.x + 60 * Charlen -1;
  R.A.y := 2 * Boxheight;  R.B.y := R.A.y + 21 * Boxheight -1;
  ClipWin := New(PEditWindow, Init(R, '',wnNoNumber));
  if ValidView(ClipWin) <> nil then begin
    ClipBoard := ClipWin^.Editor;
    ClipWin^.Hide;
    DeskTop^.Insert(ClipWin); {to insure disposing at end of program}
  end;
end;

function  TDemoApp.CreateEdWinMenu : PView;
begin
CreateEdWinMenu := New(PWindowMenu, Init(NewMenu(
      NewItem('~R~eformat', 'Ctrl+B', kbNoKey, cmBlockReformat, hcNoContext,
      NewItem('Set Right ~M~argin...', '', kbNoKey, cmSetRightMargin, hcNoContext,
      NewItem('Set ~T~ab Width...', '', kbNoKey, cmSetTabWidth, hcNoContext,
      NewItem('Toggle ~W~ordWrap', '', kbNoKey, cmToggleWordWrap, hcNoContext,
      NewItem('Toggle Auto~I~ndent', '', kbNoKey, cmIndentMode, hcNoContext,
      NewLine(
      NewItem('~S~ize/Move','Ctrl+F5', kbCtrlF5, cmResize, hcResize,
      NewItem('~Z~oom', 'F5', kbF5, cmZoom, hcZoom,
      NewLine(
      NewItem('~C~lose', 'Alt+F3', kbAltF3, cmClose, hcNoContext,
      nil))))))))))
      )));
end;

procedure TDemoApp.HandleEvent(var Event: TEvent);
  procedure ShowVersion;
  var Cmd : integer;
  begin
    Cmd := MessageBox(^C'TVGraphic '+ProgName+' ver '+Ver,
      nil, mfInformation+mfOKButton);
  end;


var
  PDir : PView;
  FInputBox : PFileDialog;
  FEdit : PEditWindow;
  Cmd : integer;
  R : TRect;
  FileName : PathStr;

   procedure OpenFileEditWindow(FName : PathStr);
    begin
      if ClipBoard = nil then CreateClipBoard;
      R.A.x := (10+WinNum) * Charlen;    R.B.x := R.A.x + 60 * Charlen -1;
      R.A.y := (2+WinNum) * Boxheight;  R.B.y := R.A.y + 21 * Boxheight -1;
      FEdit := New(PEditWindow, Init(R, FName, WinNum));
      FEdit^.Menu := CreateEdWinMenu;          {OPTIONAL}
      Inc(WinNum);
      DeskTop^.Insert(FEdit);
    end;

begin
  TProgram.HandleEvent(Event);    {usual call to ancestor method}

  if Event.What = evCommand then
  begin
    case Event.Command of
      cmNew: OpenFileEditWindow('');
      cmOpen:
        begin
          FInputBox := New(PFileDialog, Init('*.DOC', 'OPEN A FILE', '~N~ame', fdOpenButton,0));
          Cmd := DeskTop^.ExecView(FInputBox);
          FInputBox^.GetFileName(FileName);
          Dispose(FInputBox, Done);
          if (Cmd <> cmCancel) then OpenFileEditWindow(FileName);
        end;
      cmChangeDir:
        begin
          PDir := New(PChDirDialog, Init(cdNormal, 0));
          Cmd := DeskTop^.ExecView(PDir);
          Dispose(PDir, Done);
        end;
      cmVersion      : ShowVersion;
    end;
  end;
end;

procedure TDemoApp.InitMenuBar;
  {note: cmSave and cmSaveAs are handled by the GEditors unit}
var
  R: TRect;
begin
  GetExtent(R);
  MenuBar := New(PMenuBar, Init(R, NewMenu(
    NewSubMenu('~F~ile', hcNoContext, NewMenu(
      NewItem('~N~ew', '', kbNoKey, cmNew, hcNew,
      NewItem('~O~pen...', 'F3', kbF3, cmOpen, hcOpen,
      NewItem('~S~ave...', 'F2', kbF2, cmSave, hcSave,
      NewItem('Save ~a~s...', '', kbNoKey, cmSaveAs, hcSaveAs,
      NewLine(
      NewItem('~C~hange dir...', '', kbNoKey, cmChangeDir, hcChangeDir,
      NewItem('E~x~it', 'Alt+X', kbAltX, cmQuit, hcExit,
      nil)))))))),
    NewSubMenu('~E~dit', hcNoContext, NewMenu(
      StdEditMenuItems(
      nil)),
    NewSubMenu('~W~indows', hcNoContext, NewMenu(
      NewItem('~S~ize/Move','Ctrl+F5', kbCtrlF5, cmResize, hcResize,
      NewItem('~Z~oom', 'F5', kbF5, cmZoom, hcZoom,
      NewItem('~N~ext', 'F6', kbF6, cmNext, hcNext,
      NewItem('~P~revious', 'Shift+F6', kbShiftF6, cmPrev, hcPrev,
      NewItem('~C~lose', 'Alt+F3', kbAltF3, cmClose, hcClose,
      nil)))))),
    NewSubMenu('~I~nfo', hcNoContext, NewMenu(
      NewItem('~V~ersion #', '', kbNoKey, cmVersion, hcNoContext,
      nil)),
    nil))))
  )));
end;

procedure TDemoApp.InitStatusLine;
var
  R: TRect;
begin
  GetExtent(R);
  R.A.Y := R.B.Y - Boxheight+2;
  StatusLine :=   New(PStatusLine, Init(R,
    NewStatusDef(0, $FFFF,
      NewStatusKey('~F6~ Next', kbF6, cmNext,
      NewStatusKey('~Shift+F6~ Prev', kbShiftF6, cmPrev,
      NewStatusKey('~Alt-X~ Exit', kbAltX, cmQuit,
      (nil)))),
       nil)));
  StatusLine^.VFont := font8x8;  {select smaller font}
end;

procedure TDemoApp.IntroScreen;
   {^C centers following text. ^M advances to next line}
begin
  MessageBox(^C+ProgName+^M^C+
      'Demonstrates the GEditors unit and pulldown window Menus.',
      nil, mfInformation + mfOKButton);
end;


var
  DemoApp: TDemoApp;
begin
  DemoApp.Init;
  DemoApp.Run;
  DemoApp.Done;
end.
